@@ -6,7 +6,7 @@
 
 void calcX_G(Matrix<double> X_design_G [], int *offid, int noff , int *ndam, int *nsire, int nind, Matrix<int> Dams_vec [], Matrix<int> Sires_vec [], int **G, int nloci, double **A, int mtype);
 
-void calcX_Gcervus(Matrix<double> X_design_G [], int *offid, int noff , int *ndam, int *nsire, int nind, Matrix<int> Dams_vec [], Matrix<int> Sires_vec [], int **G, int nloci, double **A, double E_cervus, double **E_mat, int mtype);
+void calcX_Gcervus(Matrix<double> X_design_G [], int *offid, int noff , int *ndam, int *nsire, int nind, Matrix<int> Dams_vec [], Matrix<int> Sires_vec [], int **G, int nloci, double **A, double **E_mat, int mtype, int *nall);
 
 void calcX_Gmm(Matrix<int> mmD [], Matrix<int> mmS [], int *offid, int noff , int *ndam, int *nsire, int nind, Matrix<int> Dams_vec [], Matrix<int> Sires_vec[],int **G, int nloci);
 
@@ -17,3 +17,4 @@ void calcX_GS(Matrix<double> X_design_GD [], int *offid, int noff , int *nsire,
 void calcX_GcervusS(Matrix<double> X_design_GS [], Matrix<double> X_design_G [], int *offid, int noff , int *ndam, int *nsire, std::map<int, int> Dams [], int *dam);
 
 void calcX_GcervusD(Matrix<double> X_design_GD [], Matrix<double> X_design_G [], int *offid, int noff , int *ndam, int *nsire, std::map<int, int> Sires [], int *sire);
+
